/*
 * gps.h
 *
 *  Created on: 29-01-2012
 *      Author: andy
 * rewritten by S. James Remington 10/2014
 * parse GGA sentences 8/15
 */

#ifndef GPS_H_
#define GPS_H_

//global control

volatile char gps_parse_status=0; //global interrupt processing result
volatile char gps_parse_enable=0; //disable parsing

#define DEG2RAD 3.141592654/180.0
#define RAD2DEG 180.0/3.141592654

/*
GGA - essential fix data which provides 3D location and accuracy data.
 valid examples:
 $GPGGA,160002.00,4358.08251,N,12316.57649,W,2,08,0.96,191.8,M,-23.2,M,,0000*6A
 $GPGGA,123519,4807.038,N,01131.000,E,1,08,0.9,545.4,M,46.9,M,,*47

Where:
     GGA          Global Positioning System Fix Data
     123519       Fix taken at 12:35:19 UTC
     4807.038,N   Latitude 48 deg 07.038' N
     01131.000,E  Longitude 11 deg 31.000' E
     1            Fix quality: 
	 				0 = invalid
                    1 = GPS fix (SPS)
                    2 = DGPS fix
                    3 = PPS fix
			        4 = Real Time Kinematic
			        5 = Float RTK
                    6 = estimated (dead reckoning) (2.3 feature)
			        7 = Manual input mode
			        8 = Simulation mode
     08           Number of satellites being tracked
     0.9          Horizontal dilution of position
     545.4,M      Altitude, Meters, above mean sea level
     46.9,M       Height of geoid (mean sea level) above WGS84
                      ellipsoid
     (empty field) time in seconds since last DGPS update
     (empty field) DGPS station ID number
     *47          the checksum data, begins with *

*/

char char_count,token_index,chksum,parsemode;
char* ptemp;

char gga_time[10];  //some GPS units output 9 characters for time (two decimal places)
char gga_lat[13];  //five digits for deg/min, 1 for DP, 6 for decimal fraction plus 1 for zero terminator
char gga_NS[2];    //hemisphere, one character plus zero terminator
char gga_long[13]; // ditto for longitude
char gga_EW[2];
char gga_data_valid[2]; //1 digit plus zero terminator
char gga_sats[3];  //two digits, number of satellites tracked
char gga_hdop[6]; //HDOP horizontal dilution of precision
char gga_alt[7];    //altitude, m
char gga_altu[2];  //altitude units ('M' = meters) plus zero terminator
char gga_geoid[6];    //height of geoid
char gga_geoidu[2];  //geoid units
char gga_dgpsage[10];  //DGPS age field (size OK?)
char gga_dgpsref[5];  //DGPS reference station (4 digits)

char *gga_token[14];  //pointer array to above parsed data fields, GPGGA sentences

char gga_chksum[2];  //two character hex checksum, no zero terminator

// function prototypes

void gps_parse_init(int uart_baud);
char gps_parse(char ch);
char gps_latlon(long* lat, long* lon);
int  gga_num_sats(void);
void gps_disp_data(void);
long parse_degrees(char* string);

#endif /* GPS_H_ */
